package com.idea.buildManage.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.poi.excel.ExcelUtil;
import cn.hutool.poi.excel.ExcelWriter;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.idea.buildManage.mapper.ParkRoomExtendMapper;
import com.idea.buildManage.mapper.ParkRoomMapper;
import com.idea.buildManage.model.ParkRoom;
import com.idea.buildManage.model.ParkRoomExample;
import com.idea.buildManage.response.ParkRoomResponse;
import com.rockstar.common.base.BaseService;
import com.rockstar.common.conf.V2Config;
import com.rockstar.common.support.Convert;
import com.rockstar.frame.mapper.FrameDataMapper;
import com.rockstar.frame.model.FrameData;
import com.rockstar.frame.model.extend.DateTrans;
import com.rockstar.frame.model.extend.Tablepar;
import com.rockstar.system.service.SysDictService;
import com.rockstar.util.SnowflakeIdWorker;
import com.rockstar.util.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.beans.Transient;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

@Service
public class ParkRoomService implements BaseService<ParkRoom, ParkRoomExample> {

    //文件mapper
    @Autowired
    private SysDictService dictService;
    @Autowired
    private ParkRoomMapper modelMapper;
    @Autowired
    private FrameDataMapper frameDataMapper;
    @Autowired
    private ParkRoomExtendMapper extendMapper;

    /**
     * 分页查询
     *
     * @return
     */
    public PageInfo<ParkRoom> list(Tablepar tablepar, ParkRoom model, DateTrans dt) {
        PageHelper.startPage(tablepar.getPageNum(), tablepar.getPageSize());
        List<ParkRoom> list = modelMapper.selectByExample(getCondition(model, dt));
        PageInfo<ParkRoom> pageInfo = new PageInfo<>(list);
        return pageInfo;
    }

    /**
     * 查询指定
     *
     * @return
     */
    public List<ParkRoom> listAll(ParkRoom model, DateTrans dt) {
        return modelMapper.selectByExample(getCondition(model, dt));
    }

    private ParkRoomExample getCondition(ParkRoom model, DateTrans dt) {
        ParkRoomExample ex = new ParkRoomExample();
        ex.setOrderByClause("order_num asc");
        ParkRoomExample.Criteria criteria = ex.createCriteria();
        if (StringUtils.isNotEmpty(model.getBuildId())) {
            criteria.andBuildIdEqualTo(model.getBuildId());
        }
        return ex;
    }

    @Override
    public int deleteByPrimaryKey(String ids) {
        List<String> lista = Convert.toListStrArray(ids);
        ParkRoomExample example = new ParkRoomExample();
        example.createCriteria().andIdIn(lista);
        return modelMapper.deleteByExample(example);
    }

    @Override
    public ParkRoom selectByPrimaryKey(String id) {
        return modelMapper.selectByPrimaryKey(id);
    }

    @Override
    public int updateByPrimaryKeySelective(ParkRoom record) {
        record.setUpdatedAt(new Date());
        return modelMapper.updateByPrimaryKeySelective(record);
    }

    @Override
    public int updateByExampleSelective(ParkRoom record, ParkRoomExample example) {
        record.setUpdatedAt(new Date());
        return modelMapper.updateByExampleSelective(record, example);
    }

    @Override
    public int updateByExample(ParkRoom record, ParkRoomExample example) {
        record.setUpdatedAt(new Date());
        return modelMapper.updateByExample(record, example);
    }

    @Override
    public List<ParkRoom> selectByExample(ParkRoomExample example) {
        return modelMapper.selectByExample(example);
    }

    @Override
    public long countByExample(ParkRoomExample example) {
        return modelMapper.countByExample(example);
    }

    public long countByExample(ParkRoom model, DateTrans dt) {
        return modelMapper.countByExample(getCondition(model, dt));
    }

    @Override
    public int deleteByExample(ParkRoomExample example) {
        return modelMapper.deleteByExample(example);
    }

    public int insert(ParkRoom record) {

        record.setId(SnowflakeIdWorker.getUUID());
        return insertWithoutId(record);
    }

    public int edit(ParkRoom model) {

        ParkRoom room = selectByPrimaryKey(model.getId());
        return updateByPrimaryKeySelective(model);
    }

    public int insertWithoutId(ParkRoom record) {
        record.setCreatedAt(new Date());
        return modelMapper.insert(record);
    }

    @Override
    public int insertSelective(ParkRoom record) {
        record.setId(SnowflakeIdWorker.getUUID());
        record.setCreatedAt(new Date());
        return modelMapper.insertSelective(record);
    }

    /**
     * 分页查询
     *
     * @return
     */
    public PageInfo<ParkRoomResponse> listByModel(Tablepar tablepar, ParkRoomResponse model, DateTrans dt) {
        List<ParkRoomResponse> list = extendMapper.selectByModel(model);
        PageInfo<ParkRoomResponse> pageInfo = new PageInfo<>(list);
        return pageInfo;
    }

    /**
     * 分页查询
     *
     * @return
     */
    public List<ParkRoomResponse> excelList(ParkRoomResponse model, DateTrans dt) {
        List<ParkRoomResponse> list = extendMapper.selectByModel(model);
        return list;
    }


    //导出数据
    public String excelList(ParkRoomResponse model) {
        List<ParkRoomResponse> list = extendMapper.selectByModel(model);
        //组装数据
        List<List<String>> rows = new ArrayList<>();
        List<String> titleList = CollUtil.newArrayList("楼栋号|地号", "区域", "楼层", "房号", "面积", "朝向", "用途",
                "出租指导价", "当前状态", "租赁状态", "最后更新时间", "可用状态", "可用状态变更时间", "创建时间");
        rows.add(titleList);
        for (ParkRoomResponse parkRoomResponse : list) {
            List<String> row = new ArrayList<>();
            rows.add(row);
        }
        String filePath = V2Config.getProfile() + new SimpleDateFormat("yyyyMMddHHmmssSSS").format(new Date())
                + RandomUtil.randomInt(10000) + ".xlsx";
        ExcelWriter writer = ExcelUtil.getWriter(filePath);
        writer.merge(5, "房间列表");
        writer.write(rows);
        writer.close();
        FrameData frameData = new FrameData();
        String fdId = SnowflakeIdWorker.getUUID();
        frameData.setId(fdId);
        frameData.setFilePath(filePath);
        frameData.setCreatedAt(new Date());
        frameDataMapper.insertSelective(frameData);
        return "FileController/download/" + fdId;
    }

    // 获取房屋面积
    public Double getAreaByIds(String ids) {

        List<String> idList = Arrays.asList(ids.split(","));
        ParkRoomExample example = new ParkRoomExample();
        example.createCriteria().andIdIn(idList);
        List<ParkRoom> parkRooms = modelMapper.selectByExample(example);
        BigDecimal area = new BigDecimal("0");

        return area.doubleValue();
    }



}
